@@ -1,10 +1,5 @@
 <?php
-
-use MediaWiki\MediaWikiServices;
-
 /**
- * Kazakh (Қазақша) specific code.
- *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
@@ -21,14 +16,17 @@ use MediaWiki\MediaWikiServices;
  * http://www.gnu.org/copyleft/gpl.html
  *
  * @file
- * @ingroup Language
  */
+// phpcs:ignoreFile Squiz.Classes.ValidClassName.NotCamelCaps
+
+use MediaWiki\MainConfigNames;
+use MediaWiki\MediaWikiServices;
+
 /**
  * Kazakh (Қазақша)
  *
- * @ingroup Language
+ * @ingroup Languages
  */
-// phpcs:ignore Squiz.Classes.ValidClassName.NotCamelCaps
 class LanguageKk_cyrl extends Language {
 	# Convert from the nominative form of a noun to some other case
 	# Invoked with {{GRAMMAR:case|word}}
@@ -42,7 +40,8 @@ class LanguageKk_cyrl extends Language {
 	 * @return string
 	 */
 	protected function convertGrammarKk_cyrl( $word, $case ) {
-		$grammarForms = MediaWikiServices::getInstance()->getMainConfig()->get( 'GrammarForms' );
+		$grammarForms =
+			MediaWikiServices::getInstance()->getMainConfig()->get( MainConfigNames::GrammarForms );
 		if ( isset( $grammarForms['kk-kz'][$case][$word] ) ) {
 			return $grammarForms['kk-kz'][$case][$word];
 		}
@@ -66,7 +65,7 @@ class LanguageKk_cyrl extends Language {
 		$secondPerson = [ "з" ]; // 1st plural, 2nd formal
 		$thirdPerson = [ "ы", "і" ]; // 3rd
 
-		list( $wordEnding, $wordLastVowel ) = $this->lastLetter( $word, $allVowels );
+		[ $wordEnding, $wordLastVowel ] = $this->lastLetter( $word, $allVowels );
 
 		// Now convert the word
 		switch ( $case ) {
@@ -274,7 +273,8 @@ class LanguageKk_cyrl extends Language {
 	 * @return string
 	 */
 	protected function convertGrammarKk_latn( $word, $case ) {
-		$grammarForms = MediaWikiServices::getInstance()->getMainConfig()->get( 'GrammarForms' );
+		$grammarForms =
+			MediaWikiServices::getInstance()->getMainConfig()->get( MainConfigNames::GrammarForms );
 		if ( isset( $grammarForms['kk-tr'][$case][$word] ) ) {
 			return $grammarForms['kk-tr'][$case][$word];
 		}
@@ -298,7 +298,7 @@ class LanguageKk_cyrl extends Language {
 		$secondPerson = [ "z" ]; // 1st plural, 2nd formal
 		$thirdPerson = [ "ı", "i" ]; // 3rd
 
-		list( $wordEnding, $wordLastVowel ) = $this->lastLetter( $word, $allVowels );
+		[ $wordEnding, $wordLastVowel ] = $this->lastLetter( $word, $allVowels );
 
 		// Now convert the word
 		switch ( $case ) {
@@ -506,7 +506,8 @@ class LanguageKk_cyrl extends Language {
 	 * @return string
 	 */
 	protected function convertGrammarKk_arab( $word, $case ) {
-		$grammarForms = MediaWikiServices::getInstance()->getMainConfig()->get( 'GrammarForms' );
+		$grammarForms =
+			MediaWikiServices::getInstance()->getMainConfig()->get( MainConfigNames::GrammarForms );
 		if ( isset( $grammarForms['kk-cn'][$case][$word] ) ) {
 			return $grammarForms['kk-cn'][$case][$word];
 		}
@@ -530,7 +531,7 @@ class LanguageKk_cyrl extends Language {
 		$secondPerson = [ "ز" ]; // 1st plural, 2nd formal
 		$thirdPerson = [ "ى", "ٸ" ]; // 3rd
 
-		list( $wordEnding, $wordLastVowel ) = $this->lastLetter( $word, $allVowels );
+		[ $wordEnding, $wordLastVowel ] = $this->lastLetter( $word, $allVowels );
 
 		// Now convert the word
 		switch ( $case ) {
