@@ -1,38 +1,38 @@
 <?php
 
+use MediaWiki\EditPage\EditPage;
+use MediaWiki\MainConfigNames;
+use MediaWiki\MainConfigSchema;
+use MediaWiki\Request\FauxRequest;
 use MediaWiki\Revision\RevisionRecord;
 use MediaWiki\Storage\EditResult;
+use MediaWiki\Title\Title;
 use MediaWiki\User\UserIdentity;
 use Wikimedia\TestingAccessWrapper;
 
 /**
  * @group Editing
- *
  * @group Database
- *        ^--- tell jenkins this test needs the database
- *
  * @group medium
- *        ^--- tell phpunit that these test cases may take longer than 2 seconds.
  */
 class EditPageTest extends MediaWikiLangTestCase {
 
 	protected function setUp(): void {
 		parent::setUp();
 
-		$contLang = $this->getServiceContainer()->getContentLanguage();
-		$this->setContentLang( $contLang );
-
-		$this->setMwGlobals( [
-			'wgExtraNamespaces' => [
+		$this->overrideConfigValues( [
+			MainConfigNames::ExtraNamespaces => [
 				12312 => 'Dummy',
 				12313 => 'Dummy_talk',
 			],
-			'wgNamespaceContentModels' => [ 12312 => 'testing' ],
+			MainConfigNames::NamespaceContentModels => [ 12312 => 'testing' ],
+			MainConfigNames::ContentHandlers =>
+				[ 'testing' => 'DummyContentHandlerForTesting' ] +
+				MainConfigSchema::getDefaultValue( MainConfigNames::ContentHandlers ),
 		] );
-		$this->mergeMwGlobalArrayValue(
-			'wgContentHandlers',
-			[ 'testing' => 'DummyContentHandlerForTesting' ]
-		);
+
+		// Disable WAN cache to avoid edit conflicts in testUpdateNoMinor
+		$this->setMainCache( CACHE_NONE );
 	}
 
 	/**
@@ -140,7 +140,8 @@ class EditPageTest extends MediaWikiLangTestCase {
 			$user = $this->getTestUser()->getUser();
 		}
 
-		$page = WikiPage::factory( $title );
+		$wikiPageFactory = $this->getServiceContainer()->getWikiPageFactory();
+		$page = $wikiPageFactory->newFromTitle( $title );
 
 		if ( $baseText !== null ) {
 			$content = ContentHandler::makeContent( $baseText, $title );
@@ -197,7 +198,7 @@ class EditPageTest extends MediaWikiLangTestCase {
 				"Expected result code mismatch. $message" );
 		}
 
-		$page = WikiPage::factory( $title );
+		$page = $wikiPageFactory->newFromTitle( $title );
 
 		if ( $expectedText !== null ) {
 			// check resulting page text
@@ -285,15 +286,16 @@ class EditPageTest extends MediaWikiLangTestCase {
 	) {
 		$checkId = null;
 
-		$this->setMwGlobals( 'wgHooks', [
-			'PageSaveComplete' => [ static function (
+		$this->setTemporaryHook(
+			'PageSaveComplete',
+			static function (
 				WikiPage $page, UserIdentity $user, string $summary,
 				int $flags, RevisionRecord $revisionRecord, EditResult $editResult
 			) use ( &$checkId ) {
 				$checkId = $revisionRecord->getId();
 				// types/refs checked
-			} ],
-		] );
+			}
+		);
 
 		$edit = [ 'wpTextbox1' => $editText ];
 		if ( $ignoreBlank ) {
@@ -319,15 +321,16 @@ class EditPageTest extends MediaWikiLangTestCase {
 		$desc, $pageTitle, $user, $editText, $expectedCode, $expectedText, $ignoreBlank = false
 	) {
 		$checkIds = [];
-		$this->setMwGlobals( 'wgHooks', [
-			'PageSaveComplete' => [ static function (
+		$this->setTemporaryHook(
+			'PageSaveComplete',
+			static function (
 				WikiPage $page, UserIdentity $user, string $summary,
 				int $flags, RevisionRecord $revisionRecord, EditResult $editResult
 			) use ( &$checkIds ) {
 				$checkIds[] = $revisionRecord->getId();
 				// types/refs checked
-			} ],
-		] );
+			}
+		);
 
 		wfGetDB( DB_PRIMARY )->begin( __METHOD__ );
 
@@ -367,15 +370,16 @@ class EditPageTest extends MediaWikiLangTestCase {
 	 */
 	public function testUpdatePage() {
 		$checkIds = [];
-		$this->setMwGlobals( 'wgHooks', [
-			'PageSaveComplete' => [ static function (
+		$this->setTemporaryHook(
+			'PageSaveComplete',
+			static function (
 				WikiPage $page, UserIdentity $user, string $summary,
 				int $flags, RevisionRecord $revisionRecord, EditResult $editResult
 			) use ( &$checkIds ) {
 				$checkIds[] = $revisionRecord->getId();
 				// types/refs checked
-			} ],
-		] );
+			}
+		);
 
 		$text = "one";
 		$edit = [
@@ -477,15 +481,16 @@ class EditPageTest extends MediaWikiLangTestCase {
 		$this->forceRevisionDate( $page, '20120101000000' );
 
 		$checkIds = [];
-		$this->setMwGlobals( 'wgHooks', [
-			'PageSaveComplete' => [ static function (
+		$this->setTemporaryHook(
+			'PageSaveComplete',
+			static function (
 				WikiPage $page, UserIdentity $user, string $summary,
 				int $flags, RevisionRecord $revisionRecord, EditResult $editResult
 			) use ( &$checkIds ) {
 				$checkIds[] = $revisionRecord->getId();
 				// types/refs checked
-			} ],
-		] );
+			}
+		);
 
 		wfGetDB( DB_PRIMARY )->begin( __METHOD__ );
 
@@ -674,7 +679,7 @@ hello
 
 		$elmosEdit['wpSummary'] = 'Elmo\'s edit';
 		$bertasEdit['wpSummary'] = 'Bertas\'s edit';
-		$newEdit['wpSummary'] = $newEdit['wpSummary'] ?? 'new edit';
+		$newEdit['wpSummary'] ??= 'new edit';
 
 		// first edit: Elmo
 		$page = $this->assertEdit( __METHOD__, null, 'Elmo', $elmosEdit,
@@ -876,8 +881,8 @@ hello
 			'format' => CONTENT_FORMAT_WIKITEXT,
 		] );
 
-		$this->assertFalse( $status->isOK() );
-		$this->assertEquals( EditPage::AS_NO_CHANGE_CONTENT_MODEL, $status->getValue() );
+		$this->assertStatusNotOK( $status );
+		$this->assertStatusValue( EditPage::AS_NO_CHANGE_CONTENT_MODEL, $status );
 	}
 
 	/** @covers EditPage */
@@ -903,8 +908,8 @@ hello
 			'format' => CONTENT_FORMAT_WIKITEXT,
 		] );
 
-		$this->assertFalse( $status->isOK() );
-		$this->assertEquals( EditPage::AS_NO_CHANGE_CONTENT_MODEL, $status->getValue() );
+		$this->assertStatusNotOK( $status );
+		$this->assertStatusValue( EditPage::AS_NO_CHANGE_CONTENT_MODEL, $status );
 	}
 
 	private function doEditDummyNonTextPage( array $edit ): Status {
@@ -929,7 +934,7 @@ hello
 	 */
 	public function testWatchlistExpiry( $existingExpiry, $postVal, $selected, $options ) {
 		// Set up config and fake current time.
-		$this->setMwGlobals( 'wgWatchlistExpiry', true );
+		$this->overrideConfigValue( MainConfigNames::WatchlistExpiry, true );
 		MWTimestamp::setFakeTime( '20200505120000' );
 		$user = $this->getTestUser()->getUser();
 		$this->assertTrue( $user->isRegistered() );
@@ -963,7 +968,7 @@ hello
 	}
 
 	public function provideWatchlistExpiry() {
-		$standardOptions = [ 'infinite', '1 week', '1 month', '3 months', '6 months' ];
+		$standardOptions = [ 'infinite', '1 week', '1 month', '3 months', '6 months', '1 year' ];
 		return [
 			'not watched, request nothing' => [
 				'existingExpiry' => '',
