@@ -3,6 +3,7 @@
 namespace MediaWiki\Tests\Parser;
 
 use BagOStuff;
+use CacheTime;
 use EmptyBagOStuff;
 use HashBagOStuff;
 use InvalidArgumentException;
@@ -12,6 +13,7 @@ use MediaWiki\Page\PageRecord;
 use MediaWiki\Page\PageStoreRecord;
 use MediaWiki\Page\WikiPageFactory;
 use MediaWiki\Tests\Json\JsonUnserializableSuperClass;
+use MediaWiki\Title\Title;
 use MediaWikiIntegrationTestCase;
 use MWTimestamp;
 use NullStatsdDataFactory;
@@ -22,7 +24,6 @@ use Psr\Log\LoggerInterface;
 use Psr\Log\LogLevel;
 use Psr\Log\NullLogger;
 use TestLogger;
-use Title;
 use User;
 use Wikimedia\TestingAccessWrapper;
 use WikiPage;
@@ -134,7 +135,7 @@ class ParserCacheTest extends MediaWikiIntegrationTestCase {
 		$cache = $this->createParserCache();
 		$parserOutput = $this->createDummyParserOutput();
 
-		$cache->save( $parserOutput, $this->page, ParserOptions::newCanonical( 'canonical' ), $this->cacheTime );
+		$cache->save( $parserOutput, $this->page, ParserOptions::newFromAnon(), $this->cacheTime );
 
 		$metadataFromCache = $cache->getMetadata( $this->page, ParserCache::USE_CURRENT_ONLY );
 		$this->assertNotNull( $metadataFromCache );
@@ -150,7 +151,7 @@ class ParserCacheTest extends MediaWikiIntegrationTestCase {
 	public function testGetMetadataExpired() {
 		$cache = $this->createParserCache();
 		$parserOutput = $this->createDummyParserOutput();
-		$cache->save( $parserOutput, $this->page, ParserOptions::newCanonical( 'canonical' ), $this->cacheTime );
+		$cache->save( $parserOutput, $this->page, ParserOptions::newFromAnon(), $this->cacheTime );
 
 		$this->page = $this->createPageRecord( [ 'page_touched' => $this->time + 10000 ] );
 		$this->assertNull( $cache->getMetadata( $this->page, ParserCache::USE_CURRENT_ONLY ) );
@@ -168,7 +169,7 @@ class ParserCacheTest extends MediaWikiIntegrationTestCase {
 	public function testGetMetadataOutdated() {
 		$cache = $this->createParserCache();
 		$parserOutput = $this->createDummyParserOutput();
-		$cache->save( $parserOutput, $this->page, ParserOptions::newCanonical( 'canonical' ), $this->cacheTime );
+		$cache->save( $parserOutput, $this->page, ParserOptions::newFromAnon(), $this->cacheTime );
 
 		$this->page = $this->createPageRecord( [ 'page_latest' => $this->page->getLatest() + 1 ] );
 		$this->assertNull( $cache->getMetadata( $this->page, ParserCache::USE_CURRENT_ONLY ) );
@@ -186,12 +187,12 @@ class ParserCacheTest extends MediaWikiIntegrationTestCase {
 	public function testMakeParserOutputKey() {
 		$cache = $this->createParserCache();
 
-		$options1 = ParserOptions::newCanonical( 'canonical' );
+		$options1 = ParserOptions::newFromAnon();
 		$options1->setOption( $this->getDummyUsedOptions()[0], 'value1' );
 		$key1 = $cache->makeParserOutputKey( $this->page, $options1, $this->getDummyUsedOptions() );
 		$this->assertNotNull( $key1 );
 
-		$options2 = ParserOptions::newCanonical( 'canonical' );
+		$options2 = ParserOptions::newFromAnon();
 		$options2->setOption( $this->getDummyUsedOptions()[0], 'value2' );
 		$key2 = $cache->makeParserOutputKey( $this->page, $options2, $this->getDummyUsedOptions() );
 		$this->assertNotNull( $key2 );
@@ -204,7 +205,7 @@ class ParserCacheTest extends MediaWikiIntegrationTestCase {
 	 */
 	public function testGetEmpty() {
 		$cache = $this->createParserCache();
-		$options = ParserOptions::newCanonical( 'canonical' );
+		$options = ParserOptions::newFromAnon();
 
 		$this->assertFalse( $cache->get( $this->page, $options ) );
 	}
@@ -218,7 +219,7 @@ class ParserCacheTest extends MediaWikiIntegrationTestCase {
 		$cache = $this->createParserCache();
 		$parserOutput = new ParserOutput( 'TEST_TEXT' );
 
-		$options1 = ParserOptions::newCanonical( 'canonical' );
+		$options1 = ParserOptions::newFromAnon();
 		$options1->setOption( $this->getDummyUsedOptions()[0], 'value1' );
 		$cache->save( $parserOutput, $this->page, $options1, $this->cacheTime );
 
@@ -240,11 +241,11 @@ class ParserCacheTest extends MediaWikiIntegrationTestCase {
 		$optionName = $this->getDummyUsedOptions()[0];
 		$parserOutput = new ParserOutput( 'TEST_TEXT' );
 
-		$options1 = ParserOptions::newCanonical( 'canonical' );
+		$options1 = ParserOptions::newFromAnon();
 		$options1->setOption( $optionName, 'value1' );
 		$cache->save( $parserOutput, $this->page, $options1, $this->cacheTime );
 
-		$options2 = ParserOptions::newCanonical( 'canonical' );
+		$options2 = ParserOptions::newFromAnon();
 		$options2->setOption( $optionName, 'value2' );
 		$savedOutput = $cache->get( $this->page, $options2 );
 		$this->assertInstanceOf( ParserOutput::class, $savedOutput );
@@ -264,7 +265,7 @@ class ParserCacheTest extends MediaWikiIntegrationTestCase {
 		$parserOutput = new ParserOutput( 'TEST_TEXT' );
 		$parserOutput->updateCacheExpiry( 0 );
 
-		$options1 = ParserOptions::newCanonical( 'canonical' );
+		$options1 = ParserOptions::newFromAnon();
 		$cache->save( $parserOutput, $this->page, $options1, $this->cacheTime );
 
 		$this->assertFalse( $cache->get( $this->page, $options1 ) );
@@ -281,7 +282,7 @@ class ParserCacheTest extends MediaWikiIntegrationTestCase {
 		$parserOutput = new ParserOutput( 'TEST_TEXT' );
 		$parserOutput->recordOption( 'wrapclass' );
 
-		$options = ParserOptions::newCanonical( 'canonical' );
+		$options = ParserOptions::newFromAnon();
 		$options->setOption( 'wrapclass', 'wrapwrap' );
 
 		$cache->save( $parserOutput, $this->page, $options, $this->cacheTime );
@@ -300,9 +301,9 @@ class ParserCacheTest extends MediaWikiIntegrationTestCase {
 		$parserOutput = new ParserOutput( 'TEST_TEXT' );
 		$parserOutput->recordOption( 'wrapclass' );
 
-		$cache->save( $parserOutput, $this->page, ParserOptions::newCanonical( 'canonical' ), $this->cacheTime );
+		$cache->save( $parserOutput, $this->page, ParserOptions::newFromAnon(), $this->cacheTime );
 
-		$otherOptions = ParserOptions::newCanonical( 'canonical' );
+		$otherOptions = ParserOptions::newFromAnon();
 		$otherOptions->setOption( 'wrapclass', 'wrapwrap' );
 
 		$this->assertFalse( $cache->get( $this->page, $otherOptions ) );
@@ -319,7 +320,7 @@ class ParserCacheTest extends MediaWikiIntegrationTestCase {
 		$cache = $this->createParserCache();
 		$parserOutput = new ParserOutput( 'TEST_TEXT' );
 
-		$options = ParserOptions::newCanonical( 'canonical' );
+		$options = ParserOptions::newFromAnon();
 		$options->setOption( 'wrapclass', 'wrapwrap' );
 
 		$cache->save( $parserOutput, $this->page, $options, $this->cacheTime );
@@ -337,11 +338,11 @@ class ParserCacheTest extends MediaWikiIntegrationTestCase {
 		$optionName = $this->getDummyUsedOptions()[0];
 		$parserOutput->recordOption( $optionName );
 
-		$options1 = ParserOptions::newCanonical( 'canonical' );
+		$options1 = ParserOptions::newFromAnon();
 		$options1->setOption( $optionName, 'value1' );
 		$cache->save( $parserOutput, $this->page, $options1, $this->cacheTime );
 
-		$options2 = ParserOptions::newCanonical( 'canonical' );
+		$options2 = ParserOptions::newFromAnon();
 		$options2->setOption( $optionName, 'value2' );
 		$this->assertFalse( $cache->get( $this->page, $options2 ) );
 	}
@@ -356,7 +357,7 @@ class ParserCacheTest extends MediaWikiIntegrationTestCase {
 		$parserOutput = new ParserOutput( 'TEST_TEXT' );
 		$parserOutput->updateCacheExpiry( 10 );
 
-		$options1 = ParserOptions::newCanonical( 'canonical' );
+		$options1 = ParserOptions::newFromAnon();
 		$cache->save( $parserOutput, $this->page, $options1, $this->cacheTime );
 
 		MWTimestamp::setFakeTime( $this->time + 15 * 1000 );
@@ -379,14 +380,14 @@ class ParserCacheTest extends MediaWikiIntegrationTestCase {
 		$parserOutput1 = new ParserOutput( 'TEST_TEXT1' );
 		$parserOutput1->recordOption( $optionName );
 		$parserOutput1->updateCacheExpiry( 10 );
-		$options1 = ParserOptions::newCanonical( 'canonical' );
+		$options1 = ParserOptions::newFromAnon();
 		$options1->setOption( $optionName, 'value1' );
 		$cache->save( $parserOutput1, $this->page, $options1, $this->cacheTime );
 
 		$parserOutput2 = new ParserOutput( 'TEST_TEXT2' );
 		$parserOutput2->recordOption( $optionName );
 		$parserOutput2->updateCacheExpiry( 100500600 );
-		$options2 = ParserOptions::newCanonical( 'canonical' );
+		$options2 = ParserOptions::newFromAnon();
 		$options2->setOption( $optionName, 'value2' );
 		$cache->save( $parserOutput2, $this->page, $options2, $this->cacheTime );
 
@@ -407,7 +408,7 @@ class ParserCacheTest extends MediaWikiIntegrationTestCase {
 		$cache = $this->createParserCache();
 		$parserOutput = new ParserOutput( 'TEST_TEXT' );
 
-		$options1 = ParserOptions::newCanonical( 'canonical' );
+		$options1 = ParserOptions::newFromAnon();
 		$cache->save( $parserOutput, $this->page, $options1, $this->cacheTime );
 		$this->assertInstanceOf( ParserOutput::class,
 			$cache->get( $this->page, $options1 ) );
@@ -431,14 +432,14 @@ class ParserCacheTest extends MediaWikiIntegrationTestCase {
 
 		$parserOutput1 = new ParserOutput( 'TEST_TEXT' );
 		$parserOutput1->recordOption( $optionName );
-		$options1 = ParserOptions::newCanonical( 'canonical' );
+		$options1 = ParserOptions::newFromAnon();
 		$options1->setOption( $optionName, 'value1' );
 		$cache->save( $parserOutput1, $this->page, $options1, $this->cacheTime );
 
 		$this->page = $this->createPageRecord( [ 'page_latest' => $this->page->getLatest() + 1 ] );
 		$parserOutput2 = new ParserOutput( 'TEST_TEXT' );
 		$parserOutput2->recordOption( $optionName );
-		$options2 = ParserOptions::newCanonical( 'canonical' );
+		$options2 = ParserOptions::newFromAnon();
 		$options2->setOption( $optionName, 'value2' );
 		$cache->save( $parserOutput2, $this->page, $options2, $this->cacheTime );
 
@@ -457,7 +458,7 @@ class ParserCacheTest extends MediaWikiIntegrationTestCase {
 		$cache = $this->createParserCache();
 		$parserOutput = new ParserOutput( 'TEST_TEXT' );
 
-		$options1 = ParserOptions::newCanonical( 'canonical' );
+		$options1 = ParserOptions::newFromAnon();
 		$cache->save( $parserOutput, $this->page, $options1, $this->cacheTime );
 		$this->assertInstanceOf( ParserOutput::class,
 			$cache->get( $this->page, $options1 ) );
@@ -472,7 +473,7 @@ class ParserCacheTest extends MediaWikiIntegrationTestCase {
 	 */
 	public function testRejectedByHook() {
 		$parserOutput = new ParserOutput( 'TEST_TEXT' );
-		$options = ParserOptions::newCanonical( 'canonical' );
+		$options = ParserOptions::newFromAnon();
 		$options->setOption( $this->getDummyUsedOptions()[0], 'value1' );
 
 		$wikiPageMock = $this->createMock( WikiPage::class );
@@ -485,7 +486,7 @@ class ParserCacheTest extends MediaWikiIntegrationTestCase {
 			'RejectParserCacheValue' =>
 				function ( ParserOutput $value, WikiPage $hookPage, ParserOptions $popts )
 				use ( $wikiPageMock, $parserOutput, $options ) {
-					$this->assertSame( $parserOutput, $value );
+					$this->assertEquals( $parserOutput, $value );
 					$this->assertSame( $wikiPageMock, $hookPage );
 					$this->assertSame( $options, $popts );
 					return false;
@@ -502,7 +503,7 @@ class ParserCacheTest extends MediaWikiIntegrationTestCase {
 	 */
 	public function testParserCacheSaveCompleteHook() {
 		$parserOutput = new ParserOutput( 'TEST_TEXT' );
-		$options = ParserOptions::newCanonical( 'canonical' );
+		$options = ParserOptions::newFromAnon();
 		$options->setOption( $this->getDummyUsedOptions()[0], 'value1' );
 
 		$hookContainer = $this->createHookContainer( [
@@ -533,7 +534,7 @@ class ParserCacheTest extends MediaWikiIntegrationTestCase {
 			->with( $mockPage )
 			->willReturn( $wikiPageMock );
 		$cache = $this->createParserCache( null, null, null, $wikiPageFactoryMock );
-		$this->assertFalse( $cache->get( $mockPage, ParserOptions::newCanonical( 'canonical' ) ) );
+		$this->assertFalse( $cache->get( $mockPage, ParserOptions::newFromAnon() ) );
 	}
 
 	/**
@@ -545,7 +546,7 @@ class ParserCacheTest extends MediaWikiIntegrationTestCase {
 		$page = $this->createPageRecord( [
 			'page_is_redirect' => true
 		] );
-		$this->assertFalse( $cache->get( $page, ParserOptions::newCanonical( 'canonical' ) ) );
+		$this->assertFalse( $cache->get( $page, ParserOptions::newFromAnon() ) );
 	}
 
 	/**
@@ -566,17 +567,16 @@ class ParserCacheTest extends MediaWikiIntegrationTestCase {
 		$this->createParserCache()->save(
 			new ParserOutput( null ),
 			$this->page,
-			ParserOptions::newCanonical( 'canonical' )
+			ParserOptions::newFromAnon()
 		);
 	}
 
 	public function provideCorruptData() {
-		yield 'PHP serialization, bad data' => [ false, 'bla bla' ];
-		yield 'JSON serialization, bad data' => [ true, 'bla bla' ];
-		yield 'JSON serialization, no _class_' => [ true, '{"test":"test"}' ];
-		yield 'JSON serialization, non-existing _class_' => [ true, '{"_class_":"NonExistentBogusClass"}' ];
+		yield 'JSON serialization, bad data' => [ 'bla bla' ];
+		yield 'JSON serialization, no _class_' => [ '{"test":"test"}' ];
+		yield 'JSON serialization, non-existing _class_' => [ '{"_class_":"NonExistentBogusClass"}' ];
 		$wrongInstance = new JsonUnserializableSuperClass( 'test' );
-		yield 'JSON serialization, wrong class' => [ true, json_encode( $wrongInstance->jsonSerialize() ) ];
+		yield 'JSON serialization, wrong class' => [ json_encode( $wrongInstance->jsonSerialize() ) ];
 	}
 
 	/**
@@ -588,15 +588,13 @@ class ParserCacheTest extends MediaWikiIntegrationTestCase {
 	 * @dataProvider provideCorruptData
 	 * @covers ParserCache::get
 	 * @covers ParserCache::restoreFromJson
-	 * @param bool $json
 	 * @param string $data
 	 */
-	public function testCorruptData( bool $json, string $data ) {
+	public function testCorruptData( string $data ) {
 		$cache = $this->createParserCache( null, new HashBagOStuff() );
-		$cache->setJsonSupport( $json, $json );
 		$parserOutput = new ParserOutput( 'TEST_TEXT' );
 
-		$options1 = ParserOptions::newCanonical( 'canonical' );
+		$options1 = ParserOptions::newFromAnon();
 		$cache->save( $parserOutput, $this->page, $options1, $this->cacheTime );
 
 		$outputKey = $cache->makeParserOutputKey(
@@ -624,7 +622,7 @@ class ParserCacheTest extends MediaWikiIntegrationTestCase {
 		$cache = $this->createParserCache( null, $cacheStorage );
 		$parserOutput = new ParserOutput( 'TEST_TEXT' );
 
-		$options1 = ParserOptions::newCanonical( 'canonical' );
+		$options1 = ParserOptions::newFromAnon();
 		$cache->save( $parserOutput, $this->page, $options1, $this->cacheTime );
 
 		// Mess up the metadata
@@ -641,22 +639,46 @@ class ParserCacheTest extends MediaWikiIntegrationTestCase {
 	}
 
 	/**
-	 * Test whats happen when we turn on JSON support but there
-	 * are still old entries in the cache.
+	 * Test what happens when upgrading from 1.35 or earlier,
+	 * when old cache entries do not yet use JSON.
 	 *
 	 * @covers ParserCache::get
 	 */
 	public function testMigrationToJson() {
-		$cache = $this->createParserCache();
-		$cache->setJsonSupport( false, false );
+		$bagOStuff = new HashBagOStuff();
+
+		$cache = $this->getMockBuilder( ParserCache::class )
+			->setConstructorArgs( [
+				'test',
+				$bagOStuff,
+				'19900220000000',
+				$this->createHookContainer( [] ),
+				new JsonCodec(),
+				new NullStatsdDataFactory(),
+				new NullLogger(),
+				$this->getServiceContainer()->getTitleFactory(),
+				$this->getServiceContainer()->getWikiPageFactory()
+			] )
+			->onlyMethods( [ 'convertForCache' ] )
+			->getMock();
+
+		// Emulate pre-1.36 behavior: rely on native PHP serialization.
+		// Note that backwards compatibility of the actual serialization is covered
+		// by ParserOutputTest which uses various versions of serialized data
+		// under tests/phpunit/data/ParserCache.
+		$cache->method( 'convertForCache' )->willReturnCallback(
+			static function ( CacheTime $obj, string $key ) {
+				return $obj;
+			}
+		);
 
 		$parserOutput1 = new ParserOutput( 'Lorem Ipsum' );
 
-		$options = ParserOptions::newCanonical( 'canonical' );
+		$options = ParserOptions::newFromAnon();
 		$cache->save( $parserOutput1, $this->page, $options, $this->cacheTime );
 
 		// emulate migration to JSON
-		$cache->setJsonSupport( true, true );
+		$cache = $this->createParserCache( null, $bagOStuff );
 
 		// make sure we can load non-json cache data
 		$cachedOutput = $cache->get( $this->page, $options );
@@ -672,54 +694,15 @@ class ParserCacheTest extends MediaWikiIntegrationTestCase {
 	}
 
 	/**
-	 * Test whats happen when we have to roll back from supporting JSON
-	 * to not supporting JSON.
-	 *
-	 * @covers ParserCache::get
-	 */
-	public function testRollbackFromJson() {
-		$cache = $this->createParserCache();
-		$cache->setJsonSupport( true, true );
-
-		$parserOutput1 = new ParserOutput( 'Lorem Ipsum' );
-
-		$options = ParserOptions::newCanonical( 'canonical' );
-		$cache->save( $parserOutput1, $this->page, $options, $this->cacheTime );
-
-		// emulate rolling back to not writing but still reading JSON
-		$cache->setJsonSupport( true, false );
-
-		// make sure we can load json cache data
-		$cachedOutput = $cache->get( $this->page, $options );
-		$this->assertEquals( $parserOutput1, $cachedOutput );
-
-		// emulate rolling back to not reading JSON
-		$cache->setJsonSupport( false, false );
-
-		// make sure we don't crash and burn
-		$cachedOutput = $cache->get( $this->page, $options );
-		$this->assertFalse( $cachedOutput );
-
-		// now test that the cache works without JSON
-		$parserOutput2 = new ParserOutput( 'dolor sit amet' );
-		$cache->save( $parserOutput2, $this->page, $options, $this->cacheTime );
-
-		// make sure we can load non-json cache data
-		$cachedOutput = $cache->get( $this->page, $options );
-		$this->assertEquals( $parserOutput2, $cachedOutput );
-	}
-
-	/**
-	 * @covers ParserCache::encodeAsJson
+	 * @covers ParserCache::convertForCache
 	 */
 	public function testNonSerializableJsonIsReported() {
 		$testLogger = new TestLogger( true );
 		$cache = $this->createParserCache( null, null, $testLogger );
-		$cache->setJsonSupport( true, true );
 
 		$parserOutput = $this->createDummyParserOutput();
 		$parserOutput->setExtensionData( 'test', new User() );
-		$cache->save( $parserOutput, $this->page, ParserOptions::newCanonical( 'canonical' ) );
+		$cache->save( $parserOutput, $this->page, ParserOptions::newFromAnon() );
 		$this->assertArraySubmapSame(
 			[ [ LogLevel::ERROR, 'Unable to serialize JSON' ] ],
 			$testLogger->getBuffer()
@@ -727,18 +710,18 @@ class ParserCacheTest extends MediaWikiIntegrationTestCase {
 	}
 
 	/**
-	 * @covers ParserCache::encodeAsJson
+	 * @covers ParserCache::convertForCache
 	 */
 	public function testCyclicStructuresDoNotBlowUpInJson() {
+		$this->markTestSkipped( 'Temporarily disabled: T314338' );
 		$testLogger = new TestLogger( true );
 		$cache = $this->createParserCache( null, null, $testLogger );
-		$cache->setJsonSupport( true, true );
 
 		$parserOutput = $this->createDummyParserOutput();
 		$cyclicArray = [ 'a' => 'b' ];
 		$cyclicArray['c'] = &$cyclicArray;
 		$parserOutput->setExtensionData( 'test', $cyclicArray );
-		$cache->save( $parserOutput, $this->page, ParserOptions::newCanonical( 'canonical' ) );
+		$cache->save( $parserOutput, $this->page, ParserOptions::newFromAnon() );
 		$this->assertArraySubmapSame(
 			[ [ LogLevel::ERROR, 'Unable to serialize JSON' ] ],
 			$testLogger->getBuffer()
@@ -747,18 +730,18 @@ class ParserCacheTest extends MediaWikiIntegrationTestCase {
 
 	/**
 	 * Tests that unicode characters are not \u escaped
-	 * @covers ParserCache::encodeAsJson
+	 *
+	 * @covers ParserCache::convertForCache
 	 */
 	public function testJsonEncodeUnicode() {
 		$unicodeCharacter = "Э";
 		$cache = $this->createParserCache( null, new HashBagOStuff() );
-		$cache->setJsonSupport( true, true );
 
 		$parserOutput = $this->createDummyParserOutput();
 		$parserOutput->setText( $unicodeCharacter );
-		$cache->save( $parserOutput, $this->page, ParserOptions::newCanonical( 'canonical' ) );
+		$cache->save( $parserOutput, $this->page, ParserOptions::newFromAnon() );
 		$json = $cache->getCacheStorage()->get(
-			$cache->makeParserOutputKey( $this->page, ParserOptions::newCanonical( 'canonical' ) )
+			$cache->makeParserOutputKey( $this->page, ParserOptions::newFromAnon() )
 		);
 		$this->assertStringNotContainsString( "\u003E", $json );
 		$this->assertStringContainsString( $unicodeCharacter, $json );
